package org.jfw.util.cache;

import java.lang.reflect.Type;

public interface CacheService {
    /**
     * cache a object,non expired
     * @param obj cached object     not null
     * @return  key in cache area
     * @throws JfwCacheException
     */
    String put(Object obj) throws JfwCacheException;
    /**
     * cache object with specified expire time
     * @param obj cached object   not null
     * @param expireTime  unit is millisecond    =0 ||  expireTime > System.currMilliseconds,   0  is non expired   
     *                expireTime  is deadline
     * @return
     * @throws JfwCacheException
     */
    String put(Object obj,long expireTime)throws JfwCacheException;
    
    
    /**
     * cache a object,non expired
     * @param key 
     * @param obj cached object     not null
     * @return  key in cache area
     * @throws JfwCacheException
     */
    void put(String key,Object obj) throws JfwCacheException;
    /**
     * cache object with specified expire time
     * @param key 
     * @param obj cached object   not null
     * @param expireTime  unit is millisecond    =0 ||  expireTime > System.currMilliseconds,   0  is non expired   
     *                expireTime  is deadline
     * @return
     * @throws JfwCacheException
     */
    void put(String key,Object obj,long expireTime)throws JfwCacheException;
    
    /**
     * modify expire time with a cache object ,
     * @param key  key with cache object
     * @param newExprieTime new exprie time unit is millisecond    =0 ||  expireTime > System.currMilliseconds,   0  is non expired   
     *                expireTime  is deadline
     * @return  old exprie time , -1 is not exists 
     * @throws JfwCacheException
     */
    long expire(String key,long newExprieTime) throws JfwCacheException;
    /**
     * 
     * @param key
     * @return exprie time , -1 is not exists
     * @throws JfwCacheException
     */
    long getExprieTime(String key) throws JfwCacheException;
    /**
     * 
     * @param key
     * @return
     * @throws JfwCacheException
     */
    <T> T get(String key,Class<T> clazz)throws JfwCacheException;
    <T> T get(String key,Type typeOfT) throws JfwCacheException;
    Object get(String key)throws JfwCacheException;
    
    String put(int i)throws JfwCacheException;
    String put(byte b)throws JfwCacheException;
    String put(short s)throws JfwCacheException;
    String put(long l)throws JfwCacheException;
    String put(float f)throws JfwCacheException;
    String put(double d)throws JfwCacheException;
    String put(boolean b)throws JfwCacheException;
    String put(String s)throws JfwCacheException;
    String put(String key,int i)throws JfwCacheException;
    String put(String key,byte b)throws JfwCacheException;
    String put(String ket,short s)throws JfwCacheException;
    String put(String key,long l)throws JfwCacheException;
    String put(String key,float f)throws JfwCacheException;
    String put(String key,double d)throws JfwCacheException;
    String put(String key,boolean b)throws JfwCacheException;
    String put(String key,String s)throws JfwCacheException;
    
    int get(String key,int defaultValue)throws JfwCacheException;   
    byte get(String key,byte defaultValue)throws JfwCacheException;   
    short get(String key,short defaultValue)throws JfwCacheException;  
    long get(String key,long defaultValue)throws JfwCacheException;   
    float get(String key,float defaultValue)throws JfwCacheException;  
    double get(String key,double defaultValue)throws JfwCacheException; 
    boolean get(String key,boolean defaultValue)throws JfwCacheException;
    
    
    /**
     * remove a cached object 
     * @param key
     * @return true exists , false not exists 
     * @throws JfwCacheException
     */
    boolean remove(String key)throws JfwCacheException;
    
    
    

    
}
